<?php
/**
 * Plugin Name: Function Cache
 * Description: A fragment caching framework with customizable expiration triggers
 * Version: 20150122
 * Author: Aaron Smith
 * License: GPL2
 */


defined('ABSPATH') or die("No script kiddies please!");

//Wordpress actions that will clear specific cache triggers
add_action( 'save_post', array('function_cache','flush_db_cache'));
add_action( 'deleted_post', array('function_cache','flush_db_cache'));

//Plugin Stuff
register_activation_hook(__FILE__, array('function_cache','create_db'));
add_action( 'plugins_loaded', array('function_cache','db_check') );
register_deactivation_hook( __FILE__, array('function_cache','db_destroy') );

class function_cache {
	
/************ Cache section of the class ************/

    //Public function that allows you to load a name/value cache pair into the database with expiration triggers	
	public static function load_into_cache($name,$value,$args) {
		global $wpdb;

		$post_types = null;
		$post_ids = null;	
		
		if (!empty($args['post_types'])) {$post_types = "'".implode("','",$args['post_types'])."'";}
		if (!empty($args['post_ids'])) {$post_ids = "'".implode("','",$args['post_ids'])."'";}

		$wpdb->insert($wpdb->prefix."function_cache",array("name"=>$name,"value"=>$value,"post_types"=>$post_types,"post_ids"=>$post_ids));
	}

	//Public function to check if the cache name is already set
	public static function check_if_in_cache($name) {
		if (!empty($_GET['stop_cache'])) {return;} //to disable the cache for a page for testing purposes, just add ?stop_cache=1 to the URL
		
		global $wpdb;
		$sql = $wpdb->prepare('select value from '.$wpdb->prefix."function_cache".' where name= %s limit 0,1',$name);	
		return $wpdb->get_var($sql);
	}

	//Function used by the expiration triggers to flush the cache as need be
	public static function flush_db_cache($post_id) {
		if ( wp_is_post_revision($post_id))    return; //not smart to flush the cache on mere auto revision updates!

		global $wpdb;

		$sql = $wpdb->prepare("delete from {$wpdb->prefix}function_cache where post_ids like '%%%s%%'","'$post_id'");

		$wpdb->query($sql); //so if post 33 was just create/deleted/updated, all cache values with the expiration trigger of post id 33 will be deleted

		if (get_post_type($post_id)) {
			$sql = $wpdb->prepare("delete from {$wpdb->prefix}function_cache where post_types like '%%%s%%'","'".get_post_type($post_id)."'");
			$wpdb->query($sql); //so if post 33 was of post type houseforsale, then all cache values with the trigger of posttype = houseforsale will be deleted
		}

	}

	//Function to clean out the entire cache table
	public static function empty_entire_cache() {
		global $wpdb;		
		$sql = "truncate {$wpdb->prefix}function_cache";
		$wpdd->query($sql);
	}


/************ Utility components of the class ************/

	public static $db_version = 20150122;

	//Main function to create the database
	public static function create_db() {
		global $wpdb;
		$sql = "CREATE TABLE IF NOT EXISTS ".$wpdb->prefix."function_cache (
		  id bigint(11) NOT NULL AUTO_INCREMENT,
		  name text NOT NULL,
		  value longtext NOT NULL,
		  post_types text NOT NULL,		  
		  post_ids text NOT NULL,
		  PRIMARY KEY  (id)
		) ";
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
		add_option("function_cache_db",self::$db_version);
	}	

	//Standard DB check for plugins
	public static function db_check() {
		if ( get_site_option( 'function_cache_db' ) != self::$db_version ) {
		    self::create_db();
		}
	}

	//Helpful destructor function for those who want to properly delete the plugin
	public static function db_destroy() {
		global $wpdb;		
		delete_option('function_cache_db');
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}function_cache");
	}

	//Constructor not used because of static functions
	private function __construct() {}

/************end of class************/

}
